@@ -56,7 +56,10 @@ static const char sgSearchPathSep = ':';
 #endif
 
 #ifdef _WIN32
-#include <ShlObj.h> // for CSIDL
+#include <ShlObj.h>         // for CSIDL
+// TODO: replace this include file with the official <versionhelpers.h> header
+// included in the Windows 8.1 SDK
+#include "sgversionhelpers.hxx"
 
 static SGPath pathForCSIDL(int csidl, const SGPath& def)
 {
@@ -80,6 +83,42 @@ static SGPath pathForCSIDL(int csidl, const SGPath& def)
 
 	return def;
 }
+
+static SGPath pathForKnownFolder(REFKNOWNFOLDERID folderId, const SGPath& def)
+{
+    typedef HRESULT (WINAPI*PSHGKFP)(REFKNOWNFOLDERID, DWORD, HANDLE, PWSTR*);
+
+    HINSTANCE shellDll = LoadLibrary(TEXT("shell32"));
+    if (shellDll != NULL) {
+        PSHGKFP pSHGetKnownFolderPath = (PSHGKFP) GetProcAddress(shellDll, "SHGetKnownFolderPath");
+        if (pSHGetKnownFolderPath != NULL) {
+            // system call will allocate dynamic memory... which we must release when done
+            wchar_t* localFolder = 0;
+
+            if (pSHGetKnownFolderPath(folderId, KF_FLAG_DEFAULT_PATH, NULL, &localFolder) == S_OK) {
+                // copy into local memory
+                char path[MAX_PATH];
+                size_t len;
+                if (wcstombs_s(&len, path, localFolder, MAX_PATH) != S_OK) {
+                    path[0] = '\0';
+                    SG_LOG(SG_GENERAL, SG_WARN, "WCS to MBS failed");
+                }
+
+                SGPath folder_path = SGPath(path, def.getPermissionChecker());
+
+                // release dynamic memory
+                CoTaskMemFree(static_cast<void*>(localFolder));
+
+                return folder_path;
+            }
+        }
+
+        FreeLibrary(shellDll);
+    }
+
+    return def;
+}
+
 #elif __APPLE__
 
 // defined in CocoaHelpers.mm
@@ -194,7 +233,8 @@ SGPath::SGPath(const SGPath& p) :
   _exists(p._exists),
   _isDir(p._isDir),
   _isFile(p._isFile),
-  _modTime(p._modTime)
+  _modTime(p._modTime),
+  _size(p._size)
 {
 }
 
@@ -211,6 +251,7 @@ SGPath& SGPath::operator=(const SGPath& p)
   _isDir = p._isDir;
   _isFile = p._isFile;
   _modTime = p._modTime;
+  _size = p._size;
   return *this;
 }
 
@@ -244,6 +285,7 @@ SGPath::PermissionChecker SGPath::getPermissionChecker() const
 void SGPath::set_cached(bool cached)
 {
   _cacheEnabled = cached;
+  _cached = false;
 }
 
 // append another piece to the existing path
@@ -406,6 +448,7 @@ void SGPath::validate() const
     _isFile = ((S_IFREG & buf.st_mode ) !=0);
     _isDir = ((S_IFDIR & buf.st_mode ) !=0);
     _modTime = buf.st_mtime;
+    _size = buf.st_size;
   }
 
 #else
@@ -418,6 +461,7 @@ void SGPath::validate() const
     _isFile = ((S_ISREG(buf.st_mode )) != 0);
     _isDir = ((S_ISDIR(buf.st_mode )) != 0);
     _modTime = buf.st_mtime;
+    _size = buf.st_size;
   }
   
 #endif
@@ -647,6 +691,12 @@ time_t SGPath::modTime() const
     return _modTime;
 }
 
+size_t SGPath::sizeInBytes() const
+{
+    validate();
+    return _size;
+}
+
 bool SGPath::operator==(const SGPath& other) const
 {
     return (path == other.path);
@@ -704,20 +754,35 @@ SGPath SGPath::standardLocation(StandardLocation type, const SGPath& def)
   {
     case HOME:
       return home(def);
+
 #ifdef _WIN32
     case DESKTOP:
-      return pathForCSIDL(CSIDL_DESKTOPDIRECTORY, def);
+        if (IsWindowsVistaOrGreater())
+            return pathForKnownFolder(FOLDERID_Desktop, def);
+
+        return pathForCSIDL(CSIDL_DESKTOPDIRECTORY, def);
+
     case DOWNLOADS:
-      // TODO use KnownFolders
-      // http://msdn.microsoft.com/en-us/library/bb776911%28v=vs.85%29.aspx
-      if( !def.isNull() )
-        return def;
+        if (IsWindowsVistaOrGreater())
+            return pathForKnownFolder(FOLDERID_Downloads, def);
+
+        if (!def.isNull())
+            return def;
+
+        return pathForCSIDL(CSIDL_DESKTOPDIRECTORY, def);
 
-      return pathForCSIDL(CSIDL_DESKTOPDIRECTORY, def);
     case DOCUMENTS:
-      return pathForCSIDL(CSIDL_MYDOCUMENTS, def);
+        if (IsWindowsVistaOrGreater())
+            return pathForKnownFolder(FOLDERID_Documents, def);
+
+        return pathForCSIDL(CSIDL_MYDOCUMENTS, def);
+
     case PICTURES:
-      return pathForCSIDL(CSIDL_MYPICTURES, def);
+        if (IsWindowsVistaOrGreater())
+            return pathForKnownFolder(FOLDERID_Pictures, def);
+
+        return pathForCSIDL(CSIDL_MYPICTURES, def);
+
 #elif __APPLE__
       // since this is C++, we can't include NSPathUtilities.h to access the enum
       // values, so hard-coding them here (they are stable, don't worry)
@@ -756,20 +821,15 @@ SGPath SGPath::fromEnv(const char* name, const SGPath& def)
   return def;
 }
 
+//------------------------------------------------------------------------------
+SGPath SGPath::home(const SGPath& def)
+{
 #ifdef _WIN32
-//------------------------------------------------------------------------------
-SGPath SGPath::home(const SGPath& def)
-{
-  // TODO
-  return def;
-}
+    return fromEnv("USERPROFILE", def);
 #else
-//------------------------------------------------------------------------------
-SGPath SGPath::home(const SGPath& def)
-{
-  return fromEnv("HOME", def);
-}
+    return fromEnv("HOME", def);
 #endif
+}
 
 //------------------------------------------------------------------------------
 SGPath SGPath::desktop(const SGPath& def)
@@ -786,25 +846,39 @@ SGPath SGPath::documents(const SGPath& def)
 //------------------------------------------------------------------------------
 std::string SGPath::realpath() const
 {
-#if (defined(__APPLE__) && MAC_OS_X_VERSION_MAX_ALLOWED <= 1050)
-    // Workaround for Mac OS 10.5. Somehow fgfs crashes on Mac at ::realpath. 
-    // This means relative paths cannot be used on Mac OS <= 10.5
-    return path;
-#else
-  #if defined(_MSC_VER) /*for MS compilers */ || defined(_WIN32) /*needed for non MS windows compilers like MingW*/
+#if defined(_MSC_VER) /*for MS compilers */ || defined(_WIN32) /*needed for non MS windows compilers like MingW*/
     // with absPath NULL, will allocate, and ignore length
     char *buf = _fullpath( NULL, path.c_str(), _MAX_PATH );
-  #else
+#else
     // POSIX
     char* buf = ::realpath(path.c_str(), NULL);
-  #endif
-    if (!buf)
+#endif
+    if (!buf) // File does not exist: return the realpath it would have if created now
+              // (needed for fgValidatePath security)
     {
-        SG_LOG(SG_IO, SG_ALERT, "ERROR: The path '" << path << "' does not exist in the file system.");
-        return path;
+        if (path.empty()) {
+            return SGPath(".").realpath(); // current directory
+        }
+        std::string this_dir = dir();
+        if (isAbsolute() && this_dir.empty()) { // top level
+            this_dir = "/";
+        }
+        if (file() == "..") {
+            this_dir = SGPath(SGPath(this_dir).realpath()).dir();
+            if (this_dir.empty()) { // invalid path: .. above root
+                return "";
+            }
+            return SGPath(this_dir).realpath(); // use native path separator,
+                        // and handle 'existing/nonexisting/../symlink' paths
+        }
+        return SGPath(this_dir).realpath() +
+#if defined(_MSC_VER) || defined(_WIN32)
+          "\\" + file();
+#else
+          "/" + file();
+#endif
     }
     std::string p(buf);
     free(buf);
     return p;
-#endif
 }
