@@ -76,7 +76,7 @@ namespace simgear
       return PropertyBasedElementPtr();
 
     BOOST_FOREACH(PropertyBasedElementPtr el, _elements)
-      if( el->getProps()->getStringValue("name") == name )
+      if( el && el->getProps()->getStringValue("name") == name )
         return el;
 
     return PropertyBasedElementPtr();
@@ -96,7 +96,7 @@ namespace simgear
     _name_elements( name_elements ),
     _element_factory( element_factory )
   {
-    _props->setAttribute(SGPropertyNode::PRESERVE, true);
+
   }
 
   //----------------------------------------------------------------------------
@@ -140,7 +140,6 @@ namespace simgear
     }
 
     PropertyBasedElementPtr el = _element_factory(child);
-    el->setSelf( el );
     _elements[index] = el;
     elementCreated( el );
   }
