@@ -38,12 +38,14 @@
 #include <osgDB/ReaderWriter>
 #include <osgDB/ReadFile>
 
+#include <simgear/math/sg_random.h>
 #include <simgear/math/SGGeometry.hxx>
 #include <simgear/bucket/newbucket.hxx>
 #include <simgear/debug/logstream.hxx>
 #include <simgear/misc/sgstream.hxx>
 #include <simgear/scene/util/OptionsReadFileCallback.hxx>
 #include <simgear/scene/util/OsgMath.hxx>
+#include <simgear/scene/util/QuadTreeBuilder.hxx>
 #include <simgear/scene/util/RenderConstants.hxx>
 #include <simgear/scene/util/SGReaderWriterOptions.hxx>
 #include <simgear/scene/tgdb/apt_signs.hxx>
@@ -91,7 +93,7 @@ struct ReaderWriterSTG::_ModelBin {
         osg::ref_ptr<SGReaderWriterOptions> _options;
     };
     struct _ObjectStatic {
-        _ObjectStatic() : _agl(false), _proxy(false), _lon(0), _lat(0), _elev(0), _hdg(0), _pitch(0), _roll(0) { }
+        _ObjectStatic() : _agl(false), _proxy(false), _lon(0), _lat(0), _elev(0), _hdg(0), _pitch(0), _roll(0), _range(SG_OBJECT_RANGE) { }
         std::string _errorLocation;
         std::string _token;
         std::string _name;
@@ -99,6 +101,7 @@ struct ReaderWriterSTG::_ModelBin {
         bool _proxy;
         double _lon, _lat, _elev;
         double _hdg, _pitch, _roll;
+        double _range;
         osg::ref_ptr<SGReaderWriterOptions> _options;
     };
     struct _Sign {
@@ -113,48 +116,80 @@ struct ReaderWriterSTG::_ModelBin {
     };
 
     class DelayLoadReadFileCallback : public OptionsReadFileCallback {
+
+    private:
+      // QuadTreeBuilder for structuring static objects
+      struct MakeQuadLeaf {
+          osg::LOD* operator() () const { return new osg::LOD; }
+      };
+      struct AddModelLOD {
+          void operator() (osg::LOD* leaf, _ObjectStatic o) const
+          {
+            osg::ref_ptr<osg::Node> node;
+            if (o._proxy)  {
+                osg::ref_ptr<osg::ProxyNode> proxy = new osg::ProxyNode;
+                proxy->setName("proxyNode");
+                proxy->setLoadingExternalReferenceMode(osg::ProxyNode::DEFER_LOADING_TO_DATABASE_PAGER);
+                proxy->setFileName(0, o._name);
+                proxy->setDatabaseOptions(o._options.get());
+
+                // Give the node some values so the Quadtree builder has
+                // a BoundingBox to work with prior to the model being loaded.
+                proxy->setCenter(osg::Vec3f(0.0f,0.0f,0.0f));
+                proxy->setRadius(10);
+                proxy->setCenterMode(osg::ProxyNode::UNION_OF_BOUNDING_SPHERE_AND_USER_DEFINED);
+                node = proxy;
+            } else {
+                node = osgDB::readRefNodeFile(o._name, o._options.get());
+                if (!node.valid()) {
+                    SG_LOG(SG_TERRAIN, SG_ALERT, o._errorLocation << ": Failed to load "
+                           << o._token << " '" << o._name << "'");
+                    return;
+                }
+            }
+            if (SGPath(o._name).lower_extension() == "ac")
+                node->setNodeMask(~simgear::MODELLIGHT_BIT);
+
+            osg::Matrix matrix;
+            matrix = makeZUpFrame(SGGeod::fromDegM(o._lon, o._lat, o._elev));
+            matrix.preMultRotate(osg::Quat(SGMiscd::deg2rad(o._hdg), osg::Vec3(0, 0, 1)));
+            matrix.preMultRotate(osg::Quat(SGMiscd::deg2rad(o._pitch), osg::Vec3(0, 1, 0)));
+            matrix.preMultRotate(osg::Quat(SGMiscd::deg2rad(o._roll), osg::Vec3(1, 0, 0)));
+
+            osg::MatrixTransform* matrixTransform;
+            matrixTransform = new osg::MatrixTransform(matrix);
+            matrixTransform->setName("rotateStaticObject");
+            matrixTransform->setDataVariance(osg::Object::STATIC);
+            matrixTransform->addChild(node.get());
+
+            leaf->addChild(matrixTransform, 0, o._range);
+          }
+      };
+      struct GetModelLODCoord {
+          GetModelLODCoord() {}
+          GetModelLODCoord(const GetModelLODCoord& rhs)
+          {}
+          osg::Vec3 operator() (const _ObjectStatic& o) const
+          {
+              SGVec3d coord;
+              SGGeodesy::SGGeodToCart(SGGeod::fromDegM(o._lon, o._lat, o._elev), coord);
+              return toOsg(coord);
+          }
+      };
+      typedef QuadTreeBuilder<osg::LOD*, _ObjectStatic, MakeQuadLeaf, AddModelLOD,
+                              GetModelLODCoord>  STGObjectsQuadtree;
+
+
     public:
         virtual osgDB::ReaderWriter::ReadResult
         readNode(const std::string&, const osgDB::Options*)
         {
-            osg::ref_ptr<osg::Group> group = new osg::Group;
+            STGObjectsQuadtree quadtree((GetModelLODCoord()), (AddModelLOD()));
+            quadtree.buildQuadTree(_objectStaticList.begin(), _objectStaticList.end());
+            osg::ref_ptr<osg::Group> group = quadtree.getRoot();
             group->setName("STG-group-A");
             group->setDataVariance(osg::Object::STATIC);
-            
-            for (std::list<_ObjectStatic>::iterator i = _objectStaticList.begin(); i != _objectStaticList.end(); ++i) {
-                osg::ref_ptr<osg::Node> node;
-                if (i->_proxy)  {
-                    osg::ref_ptr<osg::ProxyNode> proxy = new osg::ProxyNode;
-                    proxy->setName("proxyNode");
-                    proxy->setLoadingExternalReferenceMode(osg::ProxyNode::DEFER_LOADING_TO_DATABASE_PAGER);
-                    proxy->setFileName(0, i->_name);
-                    proxy->setDatabaseOptions(i->_options.get());
-                    node = proxy;
-                } else {
-                    node = osgDB::readRefNodeFile(i->_name, i->_options.get());
-                    if (!node.valid()) {
-                        SG_LOG(SG_TERRAIN, SG_ALERT, i->_errorLocation << ": Failed to load "
-                               << i->_token << " '" << i->_name << "'");
-                        continue;
-                    }
-                }
-                if (SGPath(i->_name).lower_extension() == "ac")
-                    node->setNodeMask(~simgear::MODELLIGHT_BIT);
-                
-                osg::Matrix matrix;
-                matrix = makeZUpFrame(SGGeod::fromDegM(i->_lon, i->_lat, i->_elev));
-                matrix.preMultRotate(osg::Quat(SGMiscd::deg2rad(i->_hdg), osg::Vec3(0, 0, 1)));
-                matrix.preMultRotate(osg::Quat(SGMiscd::deg2rad(i->_pitch), osg::Vec3(0, 1, 0)));
-                matrix.preMultRotate(osg::Quat(SGMiscd::deg2rad(i->_roll), osg::Vec3(1, 0, 0)));
-                
-                osg::MatrixTransform* matrixTransform;
-                matrixTransform = new osg::MatrixTransform(matrix);
-                matrixTransform->setName("positionStaticObject");
-                matrixTransform->setDataVariance(osg::Object::STATIC);
-                matrixTransform->addChild(node.get());
-                group->addChild(matrixTransform);
-            }
-            
+
             simgear::AirportSignBuilder signBuilder(_options->getMaterialLib(), _bucket.get_center());
             for (std::list<_Sign>::iterator i = _signList.begin(); i != _signList.end(); ++i)
                 signBuilder.addSign(SGGeod::fromDegM(i->_lon, i->_lat, i->_elev), i->_hdg, i->_name, i->_size);
@@ -164,6 +199,7 @@ struct ReaderWriterSTG::_ModelBin {
             return group.release();
         }
         
+        mt _seed;
         std::list<_ObjectStatic> _objectStaticList;
         std::list<_Sign> _signList;
         
@@ -173,6 +209,7 @@ struct ReaderWriterSTG::_ModelBin {
     };
     
     _ModelBin() :
+        _object_range(SG_OBJECT_RANGE),
         _foundBase(false)
     { }
 
@@ -257,6 +294,12 @@ struct ReaderWriterSTG::_ModelBin {
     
         std::string filePath = osgDB::getFilePath(absoluteFileName);
 
+        // Bucket provides a consistent seed
+        // so we have consistent set of pseudo-random numbers for each STG file
+        mt seed;
+        int bucket = atoi(SGPath(absoluteFileName).file_base().c_str());
+        mt_init(&seed, bucket);
+
         // do only load airport btg files.
         bool onlyAirports = options->getPluginStringData("SimGear::FG_ONLY_AIRPORTS") == "ON";
         // do only load terrain btg files
@@ -266,7 +309,7 @@ struct ReaderWriterSTG::_ModelBin {
             // read a line
             std::string line;
             std::getline(stream, line);
-            
+
             // strip comments
             std::string::size_type hash_pos = line.find('#');
             if (hash_pos != std::string::npos)
@@ -288,7 +331,13 @@ struct ReaderWriterSTG::_ModelBin {
             
             SGPath path = filePath;
             path.append(name);
-            
+
+            // Determine an appropriate range for the object, which has some randomness
+            double range = _object_range;
+            double lrand = mt_rand(&seed);
+            if      (lrand < 0.1) range = range * 2.0;
+            else if (lrand < 0.4) range = range * 1.5;
+
             if (token == "OBJECT_BASE") {
                 // Load only once (first found)
                 SG_LOG( SG_TERRAIN, SG_BULK, "    " << token << " " << name );
@@ -329,6 +378,7 @@ struct ReaderWriterSTG::_ModelBin {
                         obj._agl = (token == "OBJECT_STATIC_AGL");
                         obj._proxy = true;
                         in >> obj._lon >> obj._lat >> obj._elev >> obj._hdg >> obj._pitch >> obj._roll;
+                        obj._range = range;
                         obj._options = opt;
                         _objectStaticList.push_back(obj);
                     }
@@ -348,6 +398,7 @@ struct ReaderWriterSTG::_ModelBin {
                         obj._agl = (token == "OBJECT_SHARED_AGL");
                         obj._proxy = false;
                         in >> obj._lon >> obj._lat >> obj._elev >> obj._hdg >> obj._pitch >> obj._roll;
+                        obj._range = range;
                         obj._options = opt;
                         _objectStaticList.push_back(obj);
                     }
@@ -377,6 +428,9 @@ struct ReaderWriterSTG::_ModelBin {
         osg::ref_ptr<SGReaderWriterOptions> options;
         options = SGReaderWriterOptions::copyOrCreate(opt);
 
+        // Determine object ranges
+        _object_range = atof(options->getPluginStringData("SimGear::ROUGH_LOD_RANGE").c_str());
+
         osg::ref_ptr<osg::Group> terrainGroup = new osg::Group;
         terrainGroup->setDataVariance(osg::Object::STATIC);
         terrainGroup->setName("terrain");
@@ -440,12 +494,15 @@ struct ReaderWriterSTG::_ModelBin {
             pagedLOD->setDatabaseOptions(callbackOptions.get());
             
             pagedLOD->setFileName(pagedLOD->getNumChildren(), "Dummy name - use the stored data in the read file callback");
-            pagedLOD->setRange(pagedLOD->getNumChildren(), 0, 30000);
+
+            // Objects may end up displayed up to 2x the object range.
+            pagedLOD->setRange(pagedLOD->getNumChildren(), 0, 2.0 * _object_range + SG_TILE_RADIUS);
             
             return pagedLOD;
         }
     }
     
+    double _object_range;
     bool _foundBase;
     std::list<_Object> _objectList;
     std::list<_ObjectStatic> _objectStaticList;
