@@ -22,10 +22,7 @@
 //
 // $Id$
 
-#ifdef HAVE_CONFIG_H
-#  include <simgear_config.h>
-#endif
-
+#include <simgear_config.h>
 #include <simgear/compiler.h>
 
 #ifdef HAVE_WINDOWS_H
@@ -50,7 +47,7 @@
 #include <string>
 #include <map>
 
-#include <simgear/compiler.h>
+#include <simgear/version.h>
 
 #include "terrasync.hxx"
 
@@ -63,14 +60,17 @@
 #include <simgear/props/props_io.hxx>
 #include <simgear/io/HTTPClient.hxx>
 #include <simgear/io/SVNRepository.hxx>
+#include <simgear/io/HTTPRepository.hxx>
+#include <simgear/io/DNSClient.hxx>
 #include <simgear/structure/exception.hxx>
+#include <simgear/math/sg_random.h>
 
 static const bool svn_built_in_available = true;
 
 using namespace simgear;
 using namespace std;
 
-const char* rsync_cmd = 
+const char* rsync_cmd =
         "rsync --verbose --archive --delete --perms --owner --group";
 
 const char* svn_options =
@@ -121,30 +121,30 @@ public:
         SharedModels,
         AIData
     };
-    
+
     enum Status
     {
         Invalid = 0,
         Waiting,
         Cached, ///< using already cached result
         Updated,
-        NotFound, 
+        NotFound,
         Failed
     };
-    
+
     SyncItem() :
         _dir(),
         _type(Stop),
         _status(Invalid)
     {
     }
-    
+
     SyncItem(string dir, Type ty) :
         _dir(dir),
         _type(ty),
         _status(Waiting)
     {}
-        
+
     string _dir;
     Type _type;
     Status _status;
@@ -164,14 +164,14 @@ public:
         isNewDirectory(false),
         busy(false)
     {}
-    
+
     SyncItem currentItem;
     bool isNewDirectory;
     std::queue<SyncItem> queue;
-    std::auto_ptr<SVNRepository> repository;
+    std::auto_ptr<AbstractRepository> repository;
     SGTimeStamp stamp;
     bool busy; ///< is the slot working or idle
-    
+
     unsigned int nextWarnTimeout;
 };
 
@@ -193,12 +193,49 @@ static unsigned int syncSlotForType(SyncItem::Type ty)
         return SYNC_SLOT_SHARED_DATA;
     case SyncItem::AIData:
         return SYNC_SLOT_AI_DATA;
-        
+
     default:
         return SYNC_SLOT_SHARED_DATA;
     }
 }
 
+///////////////////////////////////////////////////////////////////////////////
+// Base server query
+///////////////////////////////////////////////////////////////////////////////
+
+class ServerSelectQuery : public HTTP::Request
+{
+public:
+    ServerSelectQuery() :
+        HTTP::Request("http://scenery.flightgear.org/svn-server", "GET")
+    {
+    }
+
+    std::string svnUrl() const
+    {
+        std::string s = simgear::strutils::strip(m_url);
+        if (s.at(s.length() - 1) == '/') {
+            s = s.substr(0, s.length() - 1);
+        }
+
+        return s;
+    }
+protected:
+    virtual void gotBodyData(const char* s, int n)
+    {
+        m_url.append(std::string(s, n));
+    }
+
+    virtual void onFail()
+    {
+        SG_LOG(SG_TERRASYNC, SG_ALERT, "Failed to query TerraSync SVN server");
+        HTTP::Request::onFail();
+    }
+
+private:
+    std::string m_url;
+};
+
 
 ///////////////////////////////////////////////////////////////////////////////
 // SGTerraSync::SvnThread /////////////////////////////////////////////////////
@@ -220,7 +257,11 @@ public:
 
    void   setSvnServer(string server)       { _svn_server   = stripPath(server);}
    void   setSvnDataServer(string server)   { _svn_data_server   = stripPath(server);}
-    
+   void   setHTTPServer(const std::string& server)
+   {
+      _httpServer = stripPath(server);
+   }
+
    void   setExtSvnUtility(string svn_util) { _svn_command  = simgear::strutils::strip(svn_util);}
    void   setRsyncServer(string server)     { _rsync_server = simgear::strutils::strip(server);}
    void   setLocalDir(string dir)           { _local_dir    = stripPath(dir);}
@@ -245,10 +286,10 @@ public:
    volatile int _transfer_rate;
    // kbytes, not bytes, because bytes might overflow 2^31
    volatile int _total_kb_downloaded;
-   
+
 private:
    virtual void run();
-    
+
     // external model run and helpers
     void runExternal();
     void syncPathExternal(const SyncItem& next);
@@ -259,14 +300,14 @@ private:
     void updateSyncSlot(SyncSlot& slot);
 
     // commond helpers between both internal and external models
-    
+
     SyncItem::Status isPathCached(const SyncItem& next) const;
     void initCompletedTilesPersistentCache();
     void writeCompletedTilesPersistentCache() const;
     void updated(SyncItem item, bool isNewDirectory);
     void fail(SyncItem failedItem);
     void notFound(SyncItem notFoundItem);
-    
+
     bool _use_built_in;
     HTTP::Client _http;
     SyncSlot _syncSlots[NUM_SYNC_SLOTS];
@@ -274,10 +315,10 @@ private:
    volatile bool _is_dirty;
    volatile bool _stop;
    SGBlockingDeque <SyncItem> waitingTiles;
-   
+
    TileAgeCache _completedTiles;
    TileAgeCache _notFoundItems;
-   
+
    SGBlockingDeque <SyncItem> _freshTiles;
    bool _use_svn;
    string _svn_server;
@@ -286,6 +327,7 @@ private:
    string _rsync_server;
    string _local_dir;
    SGPath _persistentCachePath;
+   string _httpServer;
 };
 
 SGTerraSync::SvnThread::SvnThread() :
@@ -306,7 +348,7 @@ SGTerraSync::SvnThread::SvnThread() :
     _stop(false),
     _use_svn(true)
 {
-    _http.setUserAgent("terrascenery-" SG_STRINGIZE(SG_VERSION));
+    _http.setUserAgent("terrascenery-" SG_STRINGIZE(SIMGEAR_VERSION));
 }
 
 void SGTerraSync::SvnThread::stop()
@@ -332,7 +374,7 @@ bool SGTerraSync::SvnThread::start()
 
     if (_local_dir=="")
     {
-        SG_LOG(SG_TERRAIN,SG_ALERT,
+        SG_LOG(SG_TERRASYNC,SG_ALERT,
                "Cannot start scenery download. Local cache directory is undefined.");
         _fail_count++;
         _stalled = true;
@@ -342,7 +384,7 @@ bool SGTerraSync::SvnThread::start()
     SGPath path(_local_dir);
     if (!path.exists())
     {
-        SG_LOG(SG_TERRAIN,SG_ALERT,
+        SG_LOG(SG_TERRASYNC,SG_ALERT,
                "Cannot start scenery download. Directory '" << _local_dir <<
                "' does not exist. Set correct directory path or create directory folder.");
         _fail_count++;
@@ -353,7 +395,7 @@ bool SGTerraSync::SvnThread::start()
     path.append("version");
     if (path.exists())
     {
-        SG_LOG(SG_TERRAIN,SG_ALERT,
+        SG_LOG(SG_TERRASYNC,SG_ALERT,
                "Cannot start scenery download. Directory '" << _local_dir <<
                "' contains the base package. Use a separate directory.");
         _fail_count++;
@@ -363,17 +405,10 @@ bool SGTerraSync::SvnThread::start()
 
     _use_svn |= _use_built_in;
 
-    if ((_use_svn)&&(_svn_server==""))
-    {
-        SG_LOG(SG_TERRAIN,SG_ALERT,
-               "Cannot start scenery download. Subversion scenery server is undefined.");
-        _fail_count++;
-        _stalled = true;
-        return false;
-    }
+
     if ((!_use_svn)&&(_rsync_server==""))
     {
-        SG_LOG(SG_TERRAIN,SG_ALERT,
+        SG_LOG(SG_TERRASYNC,SG_ALERT,
                "Cannot start scenery download. Rsync scenery server is undefined.");
         _fail_count++;
         _stalled = true;
@@ -405,7 +440,7 @@ bool SGTerraSync::SvnThread::start()
 
     // not really an alert - but we want to (always) see this message, so user is
     // aware we're downloading scenery (and using bandwidth).
-    SG_LOG(SG_TERRAIN,SG_ALERT,
+    SG_LOG(SG_TERRASYNC,SG_ALERT,
            "Starting automatic scenery download/synchronization. "
            << status
            << "Directory: '" << _local_dir << "'.");
@@ -437,7 +472,7 @@ bool SGTerraSync::SvnThread::runExternalSyncCommand(const char* dir)
         // windows command line parsing is just lovely...
         // to allow white spaces, the system call needs this:
         // ""C:\Program Files\something.exe" somearg "some other arg""
-        // Note: whitespace strings quoted by a pair of "" _and_ the 
+        // Note: whitespace strings quoted by a pair of "" _and_ the
         //       entire string needs to be wrapped by "" too.
         // The svn url needs forward slashes (/) as a path separator while
         // the local path needs windows-native backslash as a path separator.
@@ -445,7 +480,7 @@ bool SGTerraSync::SvnThread::runExternalSyncCommand(const char* dir)
 #else
     command = buf.str();
 #endif
-    SG_LOG(SG_TERRAIN,SG_DEBUG, "sync command '" << command << "'");
+    SG_LOG(SG_TERRASYNC,SG_DEBUG, "sync command '" << command << "'");
 
 #ifdef SG_WINDOWS
     // tbd: does Windows support "popen"?
@@ -460,7 +495,7 @@ bool SGTerraSync::SvnThread::runExternalSyncCommand(const char* dir)
 
     if (rc)
     {
-        SG_LOG(SG_TERRAIN,SG_ALERT,
+        SG_LOG(SG_TERRASYNC,SG_ALERT,
                "Failed to synchronize directory '" << dir << "', " <<
                "error code= " << rc);
         return false;
@@ -472,13 +507,99 @@ void SGTerraSync::SvnThread::run()
 {
     _active = true;
     initCompletedTilesPersistentCache();
-    
+
+    if (_httpServer == "automatic" ) {
+
+        //TODO: make DN and service settable from properties
+        DNS::NAPTRRequest * naptrRequest = new DNS::NAPTRRequest("terrasync.flightgear.org");
+        naptrRequest->qservice = "ws20";
+
+        naptrRequest->qflags = "U";
+        DNS::Request_ptr r(naptrRequest);
+
+        DNS::Client dnsClient;
+        dnsClient.makeRequest(r);
+        while( !r->isComplete() && !r->isTimeout() )
+          dnsClient.update(0);
+
+        if( naptrRequest->entries.empty() ) {
+            SG_LOG(SG_TERRASYNC, SG_ALERT, "ERROR: automatic terrasync http-server requested, but no DNS entry found.");
+            _httpServer = "";
+        } else {
+            // walk through responses, they are ordered by 1. order and 2. preference
+            // For now, only take entries with lowest order
+            // TODO: try all available servers in the order given by preferenc and order
+            int order = naptrRequest->entries[0]->order;
+
+            // get all servers with this order and the same (for now only lowest preference)
+            DNS::NAPTRRequest::NAPTR_list availableServers;
+                for( DNS::NAPTRRequest::NAPTR_list::const_iterator it = naptrRequest->entries.begin();
+                  it != naptrRequest->entries.end();
+                  ++it ) {
+
+                if( (*it)->order != order )
+                    continue;
+
+                string regex = (*it)->regexp;
+                if( false == simgear::strutils::starts_with( (*it)->regexp, "!^.*$!" ) ) {
+                    SG_LOG(SG_TERRASYNC,SG_WARN, "ignoring unsupported regexp: " << (*it)->regexp );
+                    continue;
+                }
+
+                if( false == simgear::strutils::ends_with( (*it)->regexp, "!" ) ) {
+                    SG_LOG(SG_TERRASYNC,SG_WARN, "ignoring unsupported regexp: " << (*it)->regexp );
+                    continue;
+                }
+
+                // always use first entry
+                if( availableServers.empty() || (*it)->preference == availableServers[0]->preference) {
+                    SG_LOG(SG_TERRASYNC,SG_DEBUG, "available server regexp: " << (*it)->regexp );
+                    availableServers.push_back( *it );
+                }
+          }
+
+          // now pick a random entry from the available servers
+          DNS::NAPTRRequest::NAPTR_list::size_type idx = sg_random() * availableServers.size();
+          _httpServer = availableServers[idx]->regexp;
+          _httpServer = _httpServer.substr( 6, _httpServer.length()-7 ); // strip search pattern and separators
+
+          SG_LOG(SG_TERRASYNC,SG_INFO, "picking entry # " << idx << ", server is " << _httpServer );
+        }
+    }
+    if( _httpServer.empty() ) { // don't resolve SVN server is HTTP server is set
+        if ( _svn_server.empty()) {
+            SG_LOG(SG_TERRASYNC,SG_INFO, "Querying closest TerraSync server");
+            ServerSelectQuery* ssq = new ServerSelectQuery;
+            HTTP::Request_ptr req = ssq;
+            _http.makeRequest(req);
+            while (!req->isComplete()) {
+                _http.update(20);
+            }
+
+            if (req->readyState() == HTTP::Request::DONE) {
+                _svn_server = ssq->svnUrl();
+                SG_LOG(SG_TERRASYNC,SG_INFO, "Closest TerraSync server:" << _svn_server);
+            } else {
+                SG_LOG(SG_TERRASYNC,SG_WARN, "Failed to query closest TerraSync server");
+            }
+        } else {
+            SG_LOG(SG_TERRASYNC,SG_INFO, "Explicit: TerraSync server:" << _svn_server);
+        }
+
+        if (_svn_server.empty()) {
+#if WE_HAVE_A_DEFAULT_SERVER_BUT_WE_DONT_THIS_URL_IS_NO_LONGER_VALID
+            // default value
+            _svn_server = "http://foxtrot.mgras.net:8080/terrascenery/trunk/data/Scenery";
+#endif
+        }
+    }
+
     if (_use_built_in) {
         runInternal();
     } else {
         runExternal();
     }
-    
+
     _active = false;
     _running = false;
     _is_dirty = true;
@@ -495,14 +616,14 @@ void SGTerraSync::SvnThread::runExternal()
         SyncItem::Status cacheStatus = isPathCached(next);
         if (cacheStatus != SyncItem::Invalid) {
             _cache_hits++;
-            SG_LOG(SG_TERRAIN, SG_DEBUG,
+            SG_LOG(SG_TERRASYNC, SG_DEBUG,
                    "Cache hit for: '" << next._dir << "'");
             next._status = cacheStatus;
             _freshTiles.push_back(next);
             _is_dirty = true;
             continue;
         }
-        
+
         syncPathExternal(next);
 
         if ((_allowed_errors >= 0)&&
@@ -520,17 +641,17 @@ void SGTerraSync::SvnThread::syncPathExternal(const SyncItem& next)
     SGPath path( _local_dir );
     path.append( next._dir );
     bool isNewDirectory = !path.exists();
-    
+
     try {
         if (isNewDirectory) {
             int rc = path.create_dir( 0755 );
             if (rc) {
-                SG_LOG(SG_TERRAIN,SG_ALERT,
+                SG_LOG(SG_TERRASYNC,SG_ALERT,
                        "Cannot create directory '" << path << "', return code = " << rc );
                 throw sg_exception("Cannot create directory for terrasync", path.str());
             }
         }
-        
+
         if (!runExternalSyncCommand(next._dir.c_str())) {
             throw sg_exception("Running external sync command failed");
         }
@@ -539,7 +660,7 @@ void SGTerraSync::SvnThread::syncPathExternal(const SyncItem& next)
         _busy = false;
         return;
     }
-    
+
     updated(next, isNewDirectory);
     _busy = false;
 }
@@ -548,25 +669,25 @@ void SGTerraSync::SvnThread::updateSyncSlot(SyncSlot &slot)
 {
     if (slot.repository.get()) {
         if (slot.repository->isDoingSync()) {
-#if 0
-            if (slot.stamp.elapsedMSec() > slot.nextWarnTimeout) {
-                SG_LOG(SG_TERRAIN, SG_INFO, "sync taking a long time:" << slot.currentItem._dir << " taken " << slot.stamp.elapsedMSec());
-                SG_LOG(SG_TERRAIN, SG_INFO, "HTTP status:" << _http.hasActiveRequests());
+#if 1
+            if (slot.stamp.elapsedMSec() > (int)slot.nextWarnTimeout) {
+                SG_LOG(SG_TERRASYNC, SG_INFO, "sync taking a long time:" << slot.currentItem._dir << " taken " << slot.stamp.elapsedMSec());
+                SG_LOG(SG_TERRASYNC, SG_INFO, "HTTP request count:" << _http.hasActiveRequests());
                 slot.nextWarnTimeout += 10000;
             }
 #endif
             return; // easy, still working
         }
-        
+
         // check result
         SVNRepository::ResultCode res = slot.repository->failure();
-        if (res == SVNRepository::SVN_ERROR_NOT_FOUND) {
+        if (res == AbstractRepository::REPO_ERROR_NOT_FOUND) {
             notFound(slot.currentItem);
-        } else if (res != SVNRepository::SVN_NO_ERROR) {
+        } else if (res != AbstractRepository::REPO_NO_ERROR) {
             fail(slot.currentItem);
         } else {
             updated(slot.currentItem, slot.isNewDirectory);
-            SG_LOG(SG_TERRAIN, SG_DEBUG, "sync of " << slot.repository->baseUrl() << " finished ("
+            SG_LOG(SG_TERRASYNC, SG_DEBUG, "sync of " << slot.repository->baseUrl() << " finished ("
                    << slot.stamp.elapsedMSec() << " msec");
         }
 
@@ -574,59 +695,78 @@ void SGTerraSync::SvnThread::updateSyncSlot(SyncSlot &slot)
         slot.busy = false;
         slot.repository.reset();
     }
-    
+
     // init and start sync of the next repository
     if (!slot.queue.empty()) {
         slot.currentItem = slot.queue.front();
         slot.queue.pop();
-        
+
         SGPath path(_local_dir);
         path.append(slot.currentItem._dir);
         slot.isNewDirectory = !path.exists();
         if (slot.isNewDirectory) {
             int rc = path.create_dir( 0755 );
             if (rc) {
-                SG_LOG(SG_TERRAIN,SG_ALERT,
+                SG_LOG(SG_TERRASYNC,SG_ALERT,
                        "Cannot create directory '" << path << "', return code = " << rc );
                 fail(slot.currentItem);
                 return;
             }
         } // of creating directory step
-        
+
         string serverUrl(_svn_server);
-        if (slot.currentItem._type == SyncItem::AIData) {
-            serverUrl = _svn_data_server;
+        if (!_httpServer.empty()) {
+          slot.repository.reset(new HTTPRepository(path, &_http));
+          serverUrl = _httpServer;
+        } else {
+          if (slot.currentItem._type == SyncItem::AIData) {
+              serverUrl = _svn_data_server;
+          }
+          slot.repository.reset(new SVNRepository(path, &_http));
         }
-        
-        slot.repository.reset(new SVNRepository(path, &_http));
+
         slot.repository->setBaseUrl(serverUrl + "/" + slot.currentItem._dir);
-        slot.repository->update();
-        
+        try {
+            slot.repository->update();
+        } catch (sg_exception& e) {
+            SG_LOG(SG_TERRASYNC, SG_INFO, "sync of " << slot.repository->baseUrl() << " failed to start with error:"
+                   << e.getFormattedMessage());
+            fail(slot.currentItem);
+            slot.busy = false;
+            slot.repository.reset();
+            return;
+        }
+
         slot.nextWarnTimeout = 20000;
         slot.stamp.stamp();
         slot.busy = true;
-        SG_LOG(SG_TERRAIN, SG_DEBUG, "sync of " << slot.repository->baseUrl() << " started, queue size is " << slot.queue.size());
+        SG_LOG(SG_TERRASYNC, SG_INFO, "sync of " << slot.repository->baseUrl() << " started, queue size is " << slot.queue.size());
     }
 }
 
 void SGTerraSync::SvnThread::runInternal()
 {
     while (!_stop) {
-        _http.update(100);
+        try {
+            _http.update(100);
+        } catch (sg_exception& e) {
+            SG_LOG(SG_TERRASYNC, SG_WARN, "failure doing HTTP update" << e.getFormattedMessage());
+        }
+
         _transfer_rate = _http.transferRateBytesPerSec();
         // convert from bytes to kbytes
         _total_kb_downloaded = static_cast<int>(_http.totalBytesDownloaded() / 1024);
-        
+
         if (_stop)
             break;
- 
+
         // drain the waiting tiles queue into the sync slot queues.
         while (!waitingTiles.empty()) {
             SyncItem next = waitingTiles.pop_front();
             SyncItem::Status cacheStatus = isPathCached(next);
             if (cacheStatus != SyncItem::Invalid) {
                 _cache_hits++;
-                SG_LOG(SG_TERRAIN, SG_DEBUG, "\nTerraSync Cache hit for: '" << next._dir << "'");
+                SG_LOG(SG_TERRASYNC, SG_DEBUG, "\nTerraSync Cache hit for: '" << next._dir << "'");
                 next._status = cacheStatus;
                 _freshTiles.push_back(next);
                 _is_dirty = true;
@@ -636,7 +776,7 @@ void SGTerraSync::SvnThread::runInternal()
             unsigned int slot = syncSlotForType(next._type);
             _syncSlots[slot].queue.push(next);
         }
-       
+
         bool anySlotBusy = false;
         // update each sync slot in turn
         for (unsigned int slot=0; slot < NUM_SYNC_SLOTS; ++slot) {
@@ -663,7 +803,7 @@ SyncItem::Status SGTerraSync::SvnThread::isPathCached(const SyncItem& next) cons
         // higher levels the cache status
         return (ii == _notFoundItems.end()) ? SyncItem::Invalid : SyncItem::NotFound;
     }
-    
+
     // check if the path still physically exists. This is needed to
     // cope with the user manipulating our cache dir
     SGPath p(_local_dir);
@@ -671,7 +811,7 @@ SyncItem::Status SGTerraSync::SvnThread::isPathCached(const SyncItem& next) cons
     if (!p.exists()) {
         return SyncItem::Invalid;
     }
-    
+
     time_t now = time(0);
     return (ii->second > now) ? SyncItem::Cached : SyncItem::Invalid;
 }
@@ -683,6 +823,8 @@ void SGTerraSync::SvnThread::fail(SyncItem failedItem)
     _fail_count++;
     failedItem._status = SyncItem::Failed;
     _freshTiles.push_back(failedItem);
+    SG_LOG(SG_TERRASYNC,SG_INFO,
+           "Failed to sync'" << failedItem._dir << "'");
     _completedTiles[ failedItem._dir ] = now + UpdateInterval::FailedAttempt;
     _is_dirty = true;
 }
@@ -693,7 +835,7 @@ void SGTerraSync::SvnThread::notFound(SyncItem item)
     // as succesful download. Important for MP models and similar so
     // we don't spam the server with lookups for models that don't
     // exist
-    
+
     time_t now = time(0);
     item._status = SyncItem::NotFound;
     _freshTiles.push_back(item);
@@ -707,9 +849,9 @@ void SGTerraSync::SvnThread::updated(SyncItem item, bool isNewDirectory)
     time_t now = time(0);
     _consecutive_errors = 0;
     _success_count++;
-    SG_LOG(SG_TERRAIN,SG_INFO,
+    SG_LOG(SG_TERRASYNC,SG_INFO,
            "Successfully synchronized directory '" << item._dir << "'");
-    
+
     item._status = SyncItem::Updated;
     if (item._type == SyncItem::Tile) {
         _updated_tile_count++;
@@ -726,17 +868,17 @@ void SGTerraSync::SvnThread::initCompletedTilesPersistentCache()
     if (!_persistentCachePath.exists()) {
         return;
     }
-    
+
     SGPropertyNode_ptr cacheRoot(new SGPropertyNode);
     time_t now = time(0);
-    
+
     try {
         readProperties(_persistentCachePath.str(), cacheRoot);
     } catch (sg_exception& e) {
-        SG_LOG(SG_TERRAIN, SG_INFO, "corrupted persistent cache, discarding");
+        SG_LOG(SG_TERRASYNC, SG_INFO, "corrupted persistent cache, discarding");
         return;
     }
-    
+
     for (int i=0; i<cacheRoot->nChildren(); ++i) {
         SGPropertyNode* entry = cacheRoot->getChild(i);
         bool isNotFound = (strcmp(entry->getName(), "not-found") == 0);
@@ -745,7 +887,7 @@ void SGTerraSync::SvnThread::initCompletedTilesPersistentCache()
         if (stamp < now) {
             continue;
         }
-        
+
         if (isNotFound) {
             _completedTiles[tileName] = stamp;
         } else {
@@ -760,12 +902,12 @@ void SGTerraSync::SvnThread::writeCompletedTilesPersistentCache() const
     if (_persistentCachePath.isNull()) {
         return;
     }
-    
+
     std::ofstream f(_persistentCachePath.c_str(), std::ios::trunc);
     if (!f.is_open()) {
         return;
     }
-    
+
     SGPropertyNode_ptr cacheRoot(new SGPropertyNode);
     TileAgeCache::const_iterator it = _completedTiles.begin();
     for (; it != _completedTiles.end(); ++it) {
@@ -773,14 +915,14 @@ void SGTerraSync::SvnThread::writeCompletedTilesPersistentCache() const
         entry->setStringValue("path", it->first);
         entry->setIntValue("stamp", it->second);
     }
-    
+
     it = _notFoundItems.begin();
     for (; it != _notFoundItems.end(); ++it) {
         SGPropertyNode* entry = cacheRoot->addChild("not-found");
         entry->setStringValue("path", it->first);
         entry->setIntValue("stamp", it->second);
     }
-    
+
     writeProperties(f, cacheRoot, true /* write_all */);
     f.close();
 }
@@ -794,9 +936,9 @@ SGTerraSync::SGTerraSync() :
     _inited(false)
 {
     _svnThread = new SvnThread();
-    _log = new BufferedLogCallback(SG_TERRAIN, SG_INFO);
+    _log = new BufferedLogCallback(SG_TERRASYNC, SG_INFO);
     _log->truncateAt(255);
-    
+
     sglog().addCallback(_log);
 }
 
@@ -819,12 +961,12 @@ void SGTerraSync::init()
     if (_inited) {
         return;
     }
-    
+
     _inited = true;
-    
+
     assert(_terraRoot);
     _terraRoot->setBoolValue("built-in-svn-available",svn_built_in_available);
-        
+
     reinit();
 }
 
@@ -841,18 +983,27 @@ void SGTerraSync::reinit()
     {
         return;
     }
-    
+
     _svnThread->stop();
 
     if (_terraRoot->getBoolValue("enabled",false))
     {
         _svnThread->setSvnServer(_terraRoot->getStringValue("svn-server",""));
+        std::string httpServer(_terraRoot->getStringValue("http-server",""));
+        _svnThread->setHTTPServer(httpServer);
         _svnThread->setSvnDataServer(_terraRoot->getStringValue("svn-data-server",""));
         _svnThread->setRsyncServer(_terraRoot->getStringValue("rsync-server",""));
         _svnThread->setLocalDir(_terraRoot->getStringValue("scenery-dir",""));
         _svnThread->setAllowedErrorCount(_terraRoot->getIntValue("max-errors",5));
         _svnThread->setUseBuiltin(_terraRoot->getBoolValue("use-built-in-svn",true));
-        _svnThread->setCachePath(SGPath(_terraRoot->getStringValue("cache-path","")));
+
+        if (httpServer.empty()) {
+            // HTTP doesn't benefit from using the persistent cache
+            _svnThread->setCachePath(SGPath(_terraRoot->getStringValue("cache-path","")));
+        } else {
+            SG_LOG(SG_TERRASYNC, SG_INFO, "HTTP repository selected, disabling persistent cache");
+        }
+
         _svnThread->setCacheHits(_terraRoot->getIntValue("cache-hit", 0));
         _svnThread->setUseSvn(_terraRoot->getBoolValue("use-svn",true));
         _svnThread->setExtSvnUtility(_terraRoot->getStringValue("ext-svn-utility","svn"));
@@ -871,7 +1022,7 @@ void SGTerraSync::bind()
     if (_bound) {
         return;
     }
-    
+
     _bound = true;
     _tiedProperties.Tie( _terraRoot->getNode("busy", true), (bool*) &_svnThread->_busy );
     _tiedProperties.Tie( _terraRoot->getNode("active", true), (bool*) &_svnThread->_active );
@@ -880,11 +1031,11 @@ void SGTerraSync::bind()
     _tiedProperties.Tie( _terraRoot->getNode("tile-count", true), (int*) &_svnThread->_updated_tile_count );
     _tiedProperties.Tie( _terraRoot->getNode("cache-hits", true), (int*) &_svnThread->_cache_hits );
     _tiedProperties.Tie( _terraRoot->getNode("transfer-rate-bytes-sec", true), (int*) &_svnThread->_transfer_rate );
-    
+
     // use kbytes here because propety doesn't support 64-bit and we might conceivably
     // download more than 2G in a single session
     _tiedProperties.Tie( _terraRoot->getNode("downloaded-kbytes", true), (int*) &_svnThread->_total_kb_downloaded );
-    
+
     _terraRoot->getNode("busy", true)->setAttribute(SGPropertyNode::WRITE,false);
     _terraRoot->getNode("active", true)->setAttribute(SGPropertyNode::WRITE,false);
     _terraRoot->getNode("update-count", true)->setAttribute(SGPropertyNode::WRITE,false);
@@ -904,7 +1055,7 @@ void SGTerraSync::unbind()
     _tiedProperties.Untie();
     _bound = false;
     _inited = false;
-    
+
     _terraRoot.clear();
     _stalledNode.clear();
     _cacheHits.clear();
@@ -919,13 +1070,13 @@ void SGTerraSync::update(double)
         {
             if (_svnThread->_stalled)
             {
-                SG_LOG(SG_TERRAIN,SG_ALERT,
+                SG_LOG(SG_TERRASYNC,SG_ALERT,
                        "Automatic scenery download/synchronization stalled. Too many errors.");
             }
             else
             {
                 // not really an alert - just always show this message
-                SG_LOG(SG_TERRAIN,SG_ALERT,
+                SG_LOG(SG_TERRASYNC,SG_ALERT,
                         "Automatic scenery download/synchronization has stopped.");
             }
             _stalledNode->setBoolValue(_svnThread->_stalled);
@@ -934,7 +1085,7 @@ void SGTerraSync::update(double)
         while (_svnThread->hasNewTiles())
         {
             SyncItem next = _svnThread->getNewTile();
-            
+
             if ((next._type == SyncItem::Tile) || (next._type == SyncItem::AIData)) {
                 _activeTileDirs.erase(next._dir);
             }
@@ -958,7 +1109,7 @@ void SGTerraSync::syncAirportsModels()
             _svnThread->request( w );
         }
     }
-    
+
     SyncItem w("Models", SyncItem::SharedModels);
     _svnThread->request( w );
 }
@@ -972,7 +1123,7 @@ void SGTerraSync::syncAreaByPath(const std::string& aPath)
         if (_activeTileDirs.find(dir) != _activeTileDirs.end()) {
             continue;
         }
-                
+
         _activeTileDirs.insert(dir);
         SyncItem w(dir, SyncItem::Tile);
         _svnThread->request( w );
@@ -991,7 +1142,7 @@ bool SGTerraSync::isTileDirPending(const std::string& sceneryDir) const
     if (!_svnThread->_running) {
         return false;
     }
-    
+
     const char* terrainobjects[3] = { "Terrain/", "Objects/",  0 };
     for (const char** tree = &terrainobjects[0]; *tree; tree++) {
         string s = *tree + sceneryDir;
@@ -1008,7 +1159,7 @@ void SGTerraSync::scheduleDataDir(const std::string& dataDir)
     if (_activeTileDirs.find(dataDir) != _activeTileDirs.end()) {
         return;
     }
-    
+
     _activeTileDirs.insert(dataDir);
     SyncItem w(dataDir, SyncItem::AIData);
     _svnThread->request( w );
@@ -1020,11 +1171,11 @@ bool SGTerraSync::isDataDirPending(const std::string& dataDir) const
     if (!_svnThread->_running) {
         return false;
     }
-    
+
     return (_activeTileDirs.find(dataDir) != _activeTileDirs.end());
 }
 
 void SGTerraSync::reposition()
 {
     // stub, remove
-}
\ No newline at end of file
+}
